import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Metadata, Http } from '@zeedhi/core';
import {
    LoadingService, TextInput, IGridEvent, Select, Modal, ModalService, Form, DialogService,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class Sm00052002DescontoMaximoServicoPerfilAcessoController {
    private headerFooterModal!: Modal;

    private modalInsereDescontoMaximo!: Modal;

    public currentRow: any = {};

    get getNomePerfilAcesso() {
        return this.currentRow.NMPERFILACESSO;
    }

    public async exibirModalInsercaoDescontoMaximoServico() {
        this.headerFooterModal?.destroy();
        this.headerFooterModal = await ModalService.createFromJson('sm_00058_000_configuracoes_proposta/sm_00052_000_desconto_maximo_servico/modal/sm_00052_002_inserir-desconto-maximo-servico-perfil-acesso', true);
        this.headerFooterModal.show();
    }

    public fecharModalInsercaoDescontoMaximoServico() {
        this.headerFooterModal.hide();
        this.headerFooterModal?.destroy();
    }

    public async abrirModoInsercaoDescontoMaximoServico() {
        await this.exibirModalInsercaoDescontoMaximoServico();
    }
    
    public setFiltroGridServico(nrperfilacesso : number) {
        const component = Metadata.getInstance<TekGrid>('basic-tek-grid-modal-editar-servico-perfil-acesso');
        component.datasource.setFilter({
            NRPERFILACESSO: nrperfilacesso,
        });
    }

    public async exibirModalServicoPerfilAcesso() {
        this.modalInsereDescontoMaximo?.destroy();
        this.modalInsereDescontoMaximo = await ModalService.createFromJson('sm_00058_000_configuracoes_proposta/sm_00052_000_desconto_maximo_servico/modal/sm_00052_002_editar_servico_perfil_acesso', true);
        this.modalInsereDescontoMaximo.show();
    }

    public async abrirModalServicoPerfilAcesso({ row } : any) {
        await this.exibirModalServicoPerfilAcesso();
        this.currentRow = row;
        this.setFiltroGridServico(row.NRPERFILACESSO);
    }

    public fecharModalServicoPerfilAcesso() {
        this.modalInsereDescontoMaximo.hide();
        this.modalInsereDescontoMaximo?.destroy();
    }

    public addRowServicoPerfilAcesso() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-modal-editar-servico-perfil-acesso');
        grid.addNewRow({ NRSEQDESCPERF: 0, NRPERFILACESSO: this.currentRow.NRPERFILACESSO }, 'start');
    }

    public insertDescMaxServicoModalPerfilAcesso({ event } : IGridEvent) {
        event!.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-perfil-acesso');
        const form = Metadata.getInstance<Form>('form');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveDescMaxServicoPerfilAcesso', [{
                NRPERFILACESSO: Metadata.getInstance<TextInput>('NRPERFILACESSO').value,
                NRSEQDESCPERF: Metadata.getInstance<TextInput>('NRSEQDESCPERF').value,
                CDSERVORCA: Metadata.getInstance<Select>('CDSERVORCA').value,
                VRPERCENTUAL: Metadata.getInstance<TextInput>('VRPERCENTUAL').value,
            }]).then(() => {
                this.fecharModalInsercaoDescontoMaximoServico();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                if (error.response.data.error === 'ERRO') {
                    Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Já existe um desconto cadastrado para o Serviço e Perfil de Acesso informado.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public insertServicoPerfilAcesso({ event, component } : any) {
        event!.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-modal-editar-servico-perfil-acesso');
        const editedrows = grid.getEditedRows();

        if (component.isGridValid()) {
            LoadingService.show();
            Http.post('/saveDescMaxServicoPerfilAcesso', editedrows).then(() => {
                grid.cancelEditedRows();
                grid.reload();
                Utilities.showSuccessMessage();
                grid.datasource.get();
            }).catch((error) => {
                if (error.response.data.error === 'ERRO') {
                    Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Já existe um desconto cadastrado para o Serviço e Perfil de Acesso informado.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Preencha todos os campos obrigatórios', 'error');
        }
    }

    public showConfirmDeleteMessage({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteDescMaxServicoModalServicoPerfilAcesso();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteDescMaxServicoModalServicoPerfilAcesso() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-modal-editar-servico-perfil-acesso');
        const gridPerfilAcesso = Metadata.getInstance<TekGrid>('basic-tek-grid-perfil-acesso');
        LoadingService.show();
        Http.post('/deleteDescMaxServicoModalServicoPerfilAcesso', grid.selectedRows).then(() => {
            grid.selectedRows = [];
            grid.cancelEditedRows();
            grid.reload();
            gridPerfilAcesso.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
